/*
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.apache.bval.util;

import java.util.OptionalInt;
import java.util.function.IntSupplier;

/**
 * @since 2.0
 */
public class LazyInt implements IntSupplier {
    private int value;
    private volatile IntSupplier init;

    public LazyInt(IntSupplier init) {
        this.init = Validate.notNull(init);
    }

    @Override
    public int getAsInt() {
        if (init != null) {
            synchronized (this) {
                if (init != null) {
                    value = init.getAsInt();
                    init = null;
                }
            }
        }
        return value;
    }

    public synchronized OptionalInt optional() {
        return init == null ? OptionalInt.of(value) : OptionalInt.empty();
    }
}
